@@ -1,6 +1,6 @@
 #  MIT License
 #
-#  Copyright (c) 2017-2020 TileDB Inc.
+#  Copyright (c) 2017-2021 TileDB Inc.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -41,6 +41,8 @@ tiledb_attr.from_ptr <- function(ptr) {
 #' that this is a _required_ parameter.
 #' @param filter_list (default filter_list("NONE")) The tiledb_attr filter_list
 #' @param ncells (default 1) The number of cells, use \code{NA} to signal variable length
+#' @param nullable (default FALSE) A logical switch whether the attribute can have missing
+#' values
 #' @param ctx tiledb_ctx object (optional)
 #' @return `tiledb_dim` object
 #' @examples
@@ -56,6 +58,7 @@ tiledb_attr <- function(name,
                         type,
                         filter_list = tiledb_filter_list(),
                         ncells = 1,
+                        nullable = FALSE,
                         ctx = tiledb_get_context()
                         ) {
     if (missing(name)) {
@@ -71,7 +74,7 @@ tiledb_attr <- function(name,
     } else if(!is(filter_list, "tiledb_filter_list")) {
         stop("filter_list argument must be a tiledb_filter_list instance")
     }
-    ptr <- libtiledb_attribute(ctx@ptr, name, type, filter_list@ptr, ncells)
+    ptr <- libtiledb_attribute(ctx@ptr, name, type, filter_list@ptr, ncells, nullable)
     new("tiledb_attr", ptr = ptr)
 }
 
@@ -128,9 +131,11 @@ setMethod("datatype", signature(object = "tiledb_attr"),
             libtiledb_attribute_get_type(object@ptr)
           })
 
-#' Returns the `tiledb_filter_list` object associated with the given `tiledb_attr`
+## Generic in ArraySchema.R
+
+#' Returns the TileDB Filter List object associated with the given TileDB Attribute
 #'
-#' @param object tiledb_attr
+#' @param object TileDB Attribute
 #' @return a tiledb_filter_list object
 #' @examples
 #' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
@@ -138,16 +143,35 @@ setMethod("datatype", signature(object = "tiledb_attr"),
 #' filter_list(attr)
 #'
 #' @export
-setMethod("filter_list", "tiledb_attr",
-          function(object) {
-            ptr <- libtiledb_attribute_get_filter_list(object@ptr)
-            return(tiledb_filter_list.from_ptr(ptr))
-          })
+setMethod("filter_list", "tiledb_attr", function(object) {
+  ptr <- libtiledb_attribute_get_filter_list(object@ptr)
+  return(tiledb_filter_list.from_ptr(ptr))
+})
 
-#' @rdname generics
+## Generic in ArraySchema.R
+
+#' Sets the TileDB Filter List for the TileDB Attribute object
+#'
+#' @param x TileDB Attribute
+#' @param value TileDB Filter List
+#' @return The modified TileDB Attribute object
+#' @export
+setReplaceMethod("filter_list", "tiledb_attr", function(x, value) {
+  x@ptr <- libtiledb_attribute_set_filter_list(x@ptr, value@ptr)
+  x
+})
+
+
+#' @rdname tiledb_attribute_get_cell_val_num
 #' @export
 setGeneric("cell_val_num", function(object) standardGeneric("cell_val_num"))
 
+#' @rdname tiledb_attribute_get_cell_val_num
+#' @export
+setMethod("cell_val_num", signature(object = "tiledb_attr"), function(object) {
+  libtiledb_attribute_get_cell_val_num(object@ptr)
+})
+
 #' Return the number of scalar values per attribute cell
 #'
 #' @param object `tiledb_attr` object
@@ -156,12 +180,34 @@ setGeneric("cell_val_num", function(object) standardGeneric("cell_val_num"))
 #' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' a1 <- tiledb_attr("a1", type = "FLOAT64", ncells = 1)
 #' cell_val_num(a1)
-#'
 #' @export
-setMethod("cell_val_num", signature(object = "tiledb_attr"),
-          function(object) {
-            libtiledb_attribute_get_cell_val_num(object@ptr)
-          })
+tiledb_attribute_get_cell_val_num <- function(object) {
+  libtiledb_attribute_get_cell_val_num(object@ptr)
+}
+
+
+
+#' @rdname tiledb_attribute_set_cell_val_num
+#' @export
+setGeneric("cell_val_num<-", function(x, value) standardGeneric("cell_val_num<-"))
+
+#' @rdname tiledb_attribute_set_cell_val_num
+#' @export
+setReplaceMethod("cell_val_num", signature("tiledb_attr"), function(x, value) {
+  libtiledb_attribute_set_cell_val_num(x@ptr, value)
+  x
+})
+
+#' Set the number of scalar values per attribute cell
+#'
+#' @param x A TileDB Attribute object
+#' @param value An integer value of number of cells
+#' @return The modified attribute is returned
+#' @export
+tiledb_attribute_set_cell_val_num <- function(x, value) {
+  libtiledb_attribute_set_cell_val_num(x@ptr, value)
+}
+
 
 #' Returns TRUE if the tiledb_dim is anonymous
 #'
@@ -186,3 +232,69 @@ is.anonymous.tiledb_attr <- function(object) {
   name <- libtiledb_attribute_get_name(object@ptr)
   nchar(name) == 0
 }
+
+
+#' Get the fill value for a TileDB Attribute
+#'
+#' @param attr A TileDB Attribute object
+#' @return The fill value for the attribute
+#' @export
+tiledb_attribute_get_fill_value <- function(attr) {
+  stopifnot(attr_object=is(attr, "tiledb_attr"))
+  libtiledb_attribute_get_fill_value(attr@ptr)
+}
+
+#' Set the fill value for a TileDB Attribute
+#'
+#' @param attr A TileDB Attribute object
+#' @param value A fill value
+#' @return \code{NULL} is returned invisibly
+#' @export
+tiledb_attribute_set_fill_value <- function(attr, value) {
+  stopifnot(attr_object=is(attr, "tiledb_attr"),
+            value_type=is.integer(value) || is.numeric(value) || is.character(value))
+  libtiledb_attribute_set_fill_value(attr@ptr, value)
+  invisible()
+}
+
+#' Check whether TileDB Attribute is variable-sized
+#'
+#' @param attr A TileDB Attribute object
+#' @return A boolean value indicating variable-size or not
+#' @export
+tiledb_attribute_is_variable_sized <- function(attr) {
+  stopifnot(attr_object=is(attr, "tiledb_attr"))
+  libtiledb_attribute_is_variable_sized(attr@ptr)
+}
+
+#' Get the TileDB Attribute cell size
+#'
+#' @param attr A TileDB Attribute object
+#' @return A numeric value with the cell size
+#' @export
+tiledb_attribute_get_cell_size <- function(attr) {
+  stopifnot(attr_object=is(attr, "tiledb_attr"))
+  libtiledb_attribute_get_cell_size(attr@ptr)
+}
+
+#' Set the TileDB Attribute Nullable flags
+#'
+#' @param attr A TileDB Attribute object
+#' @param flag A boolean flag to turn \sQuote{Nullable} on or off
+#' @return Nothing is returned
+#' @export
+tiledb_attribute_set_nullable <- function(attr, flag) {
+    stopifnot(attr_object=is(attr, "tiledb_attr"),
+              flag_boolean_not_na=is.logical(flag) & !is.na(flag))
+    libtiledb_attribute_set_nullable(attr@ptr, flag)
+}
+
+#' Get the TileDB Attribute Nullable flag value
+#'
+#' @param attr A TileDB Attribute object
+#' @return A boolean value with the \sQuote{Nullable} status
+#' @export
+tiledb_attribute_get_nullable <- function(attr) {
+    stopifnot(attr_object=is(attr, "tiledb_attr"))
+    libtiledb_attribute_get_nullable(attr@ptr)
+}
